﻿using UnityEngine;
using System;
using System.Linq;
using System.Collections.Generic;

public class HorizontalGeneration : MonoBehaviour
{
	
	public TilemapController tilemapController;

	public int width = 100;
	public int height = 100;
	public float noiseScaleX = 2;
	public float noiseScaleY = 50;

	public int offsetX = 0;
	public int offsetY = 0;

	public float frequency = 1;
	public float frequencyXNoise = 3;

	public ushort seed = 0b1100000011;

	Grid grid;

	// Start is called before the first frame update
	void Start()
	{
		Generation();
    }

    // Update is called once per frame
    void Update()
    {
		if (Input.GetKeyDown(KeyCode.Space))
		{
			tilemapController.Clear();
			Generation();
		}
	}

	private void Generation()
    {
		ushort seedX = (ushort)((ushort)(seed << 8)>>8);
		byte seedY = (byte)(seed >> 8);

		Debug.Log("Seed X: " + Convert.ToString(seedX, 2) + " Seed Y: " + Convert.ToString(seedY, 2));
		Debug.Log("NoiseScaleX: " + noiseScaleX + " NoiseScaleY: " + noiseScaleY);
		Debug.Log("OffsetX: " + offsetX + " OffsetY: " + offsetY);
		Debug.Log("FrequencyY: " + frequency + " FrequencyX: " + frequencyXNoise);
		grid = new Grid(width, height);

		int startX = 0;
		int startY = 50;

		int perlinX = 0;
		float perlinY = 0;

		int x = startX;
		int y = startY;

		int breakCounter = 0;
		Vector2Int pos = new Vector2Int(x, y);
		Vector2Int oldPos = pos;
		while (x < width-1 && breakCounter < 200000)
		{
			//Erstellen des Höhenwertes für die nächste Iteration
			float xCoord = (float)(perlinX + offsetX) / 1000 * frequency;
			float xNoise = Mathf.PerlinNoise(xCoord, (float)seedY / 0b11111111)-0.5f;
			y = startY + Mathf.RoundToInt(xNoise * noiseScaleY);

			//Erstellen des Horizontalen Wertes für die nächste Iteration
			float yCoord = (float)(perlinY + offsetY) / 1000 * frequencyXNoise;
			int noise = Mathf.RoundToInt((Mathf.PerlinNoise((float)seedX / 0b11111111, yCoord)-0.5f) * noiseScaleX*2);
			x = perlinX + noise;

			
			//Prüfen ob die Koordinaten die Beschränkungen der Welt überschreiten
			if (x < 0)
			{
				x = 0;
			}
			else if (x >= width)
			{
				x = width - 1;
			}

			if (y < 0)
			{
				y = 0;
			}
			else if (y >= height)
			{
				y = height - 1;
			}

			pos = new Vector2Int(x, y);
            //Interpolieren zwischen altem und neuem Punkt

            if (distanceToHigh(oldPos, pos))
            {
                Interpolate(oldPos, pos);
            }

            //if (y-yOld>1)
            //         {
            //	x--;
            //	grid.setCell(x, yOld + 1);
            //} else if(y-yOld < -1)
            //         {
            //	x--;
            //	grid.setCell(x, yOld - 1);
            //} 
            grid.setCell(x, y);


			oldPos = pos;
			perlinX++;
			perlinY++;
			//Debug.Log(perlinX + ":" + x);
			breakCounter++;
		}

		//floddfill

		Fill4();

		for (x = 0; x < width; x++)	
		{
			for (y = 0; y < height; y++)
			{
				if (grid.getCell(x, y) == 1)
				{
					tilemapController.spawnTile(x, y, TileType.GRASS);
				}
				else if (grid.getCell(x,y) == 2)
				{
					tilemapController.spawnTile(x, y, TileType.DIRT);
				}
			}

		}
	}

    void Fill4(int x = 0, int y = 0, int oldTile = 0, int newTile = 2)
    {
		if (!isPlaceable(x, y, oldTile, newTile))
			return;
        if (grid.getCell(x, y) == oldTile)
        {
            grid.setCell(x, y, newTile);
			Fill4(x + 1, y, oldTile, newTile);  // rechts
			Fill4(x, y + 1, oldTile, newTile);  // oben
			Fill4(x - 1, y, oldTile, newTile);  // links
			//Fill4(x, y - 1, oldTile, newTile);  // unten
		}
    }

	bool isPlaceable(int x = 0, int y = 0, int oldTile = 0, int newTile = 2)
    {
		if (x < 0 || x > width - 1 || y < 0 || y > height - 1 || grid.getCell(x, y) == newTile)
		{
			return false;
		}
		return true;
	}


    void Interpolate(Vector2Int start, Vector2Int target)
    {

		//Richtung bestimmen
		Vector2 direction = target - start;
		direction.Normalize();




		//Einmal Interpolieren
		Vector2Int directionInt = new Vector2Int(Mathf.RoundToInt(direction.x), Mathf.RoundToInt(direction.y));
		start += directionInt;

		grid.setCell(start.x, start.y);


		//Prüfen ob fertig
		if(distanceToHigh(start, target))
        {
			Interpolate(start, target);
        }

		//ggf Recursiv ausführen
	}

	private bool distanceToHigh(Vector2 start, Vector2 target)
    {
		float distance = Mathf.Sqrt(Mathf.Pow((start.x - target.x), 2) + Mathf.Pow((start.y - target.y), 2));
		return distance > Mathf.Sqrt(2);
	}
}
