﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;



public class TerrainGenerationEasyCosineInterpolated : Generation
{
    public override int[] GetHeightValues(int width, int maxHeight,int minHeight, int seed)
    {
        int[] heightValues = DefaultGenerator.GetRandomNoise(seed, width, maxHeight, minHeight);

        int[] newHeightValues = new int[width];
        System.Array.Copy(heightValues, newHeightValues, width);

        bool finished = false;
        int index = 0;
        for (int i = 1; i < heightValues.Length && !finished; i++)
        {
            newHeightValues[index] = heightValues[i - 1];
            index++;
            int diff = heightValues[i] - heightValues[i - 1];
            if (diff > 1 || diff < -1)
            {
                bool ascend = true;
                if (diff < 0)
                {
                    ascend = false;
                    diff = Mathf.Abs(diff);
                }
                int[] interpolatedValues = new int[diff - 1];
                for (int j = 1; j < diff; j++)
                {
                    int temp = Mathf.RoundToInt((1 - Mathf.Cos(Mathf.PI * j / diff)) * diff / 2);
                    if (ascend)
                    { // Aufstieg 
                        interpolatedValues[j - 1] = heightValues[i - 1] + temp;
                    }
                    else
                    { // Abstieg
                        interpolatedValues[j - 1] = heightValues[i - 1] - temp;
                    }
                }
                int indexStart = index;
                while (index < indexStart + diff - 1 && !finished)
                {
                    if (index < width && !finished)
                    {
                        newHeightValues[index] = interpolatedValues[index - indexStart];
                        index++;
                    }
                    if (index >= width)
                    {
                        finished = true;
                    }
                    
                }
            }
            
        }
        return newHeightValues;
    }
}
